@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2015 The git2r contributors
+ *  Copyright (C) 2013-2017 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -43,23 +43,29 @@ void git2r_tag_init(git_tag *source, SEXP repo, SEXP dest)
     const git_oid *oid;
     char sha[GIT_OID_HEXSZ + 1];
     char target[GIT_OID_HEXSZ + 1];
+    SEXP s_sha = Rf_install("sha");
+    SEXP s_message = Rf_install("message");
+    SEXP s_name = Rf_install("name");
+    SEXP s_tagger = Rf_install("tagger");
+    SEXP s_target = Rf_install("target");
+    SEXP s_repo = Rf_install("repo");
 
     oid = git_tag_id(source);
     git_oid_tostr(sha, sizeof(sha), oid);
-    SET_SLOT(dest, Rf_install("sha"), mkString(sha));
+    SET_SLOT(dest, s_sha, mkString(sha));
 
-    SET_SLOT(dest, Rf_install("message"), mkString(git_tag_message(source)));
-    SET_SLOT(dest, Rf_install("name"), mkString(git_tag_name(source)));
+    SET_SLOT(dest, s_message, mkString(git_tag_message(source)));
+    SET_SLOT(dest, s_name, mkString(git_tag_name(source)));
 
     tagger = git_tag_tagger(source);
     if (tagger)
-        git2r_signature_init(tagger, GET_SLOT(dest, Rf_install("tagger")));
+        git2r_signature_init(tagger, GET_SLOT(dest, s_tagger));
 
     oid = git_tag_target_id(source);
     git_oid_tostr(target, sizeof(target), oid);;
-    SET_SLOT(dest, Rf_install("target"), mkString(target));
+    SET_SLOT(dest, s_target, mkString(target));
 
-    SET_SLOT(dest, Rf_install("repo"), repo);
+    SET_SLOT(dest, s_repo, repo);
 }
 
 /**
@@ -197,7 +203,7 @@ static int git2r_tag_foreach_cb(const char *name, git_oid *oid, void *payload)
     /* Check if we have a list to populate */
     if (R_NilValue != cb_data->tags) {
         int skip = 0;
-        SEXP item;
+        SEXP item, tag;
 
         err = git_object_lookup(&object, cb_data->repository, oid, GIT_OBJ_ANY);
         if (err)
@@ -236,13 +242,14 @@ static int git2r_tag_foreach_cb(const char *name, git_oid *oid, void *payload)
             git2r_error(__func__, NULL, git2r_err_object_type, NULL);
         }
 
-
         if (git__prefixcmp(name, "refs/tags/") == 0)
             skip = strlen("refs/tags/");
+        PROTECT(tag = mkChar(name + skip));
         SET_STRING_ELT(
             getAttrib(cb_data->tags, R_NamesSymbol),
             cb_data->n,
-            mkChar(name + skip));
+            tag);
+        UNPROTECT(1);
 
         if (object)
             git_object_free(object);
